<?php
/**
 * Created by PhpStorm.
 * User: ep
 * Date: 12/23/15
 * Time: 6:33 PM
 */


namespace qfq;

use qfq;
use qfq\BuildFormPlain;
use qfq\BuildFormTable;
use qfq\BuildFormBootstrap;
use qfq\UserException;
use qfq\CodeException;
use qfq\DbException;
//use qfq\helper;
use qfq\Store;


require_once(__DIR__ . '/../qfq/store/Store.php');
require_once(__DIR__ . '/../qfq/Constants.php');
require_once(__DIR__ . '/../qfq/helper/KeyValueStringParser.php');
require_once(__DIR__ . '/../qfq/helper/HelperFormElement.php');
require_once(__DIR__ . '/../qfq/exceptions/UserException.php');
require_once(__DIR__ . '/../qfq/exceptions/CodeException.php');
require_once(__DIR__ . '/../qfq/exceptions/DbException.php');
require_once(__DIR__ . '/../qfq/Database.php');
require_once(__DIR__ . '/../qfq/Evaluate.php');
require_once(__DIR__ . '/../qfq/BuildFormPlain.php');
require_once(__DIR__ . '/../qfq/BuildFormTable.php');
require_once(__DIR__ . '/../qfq/BuildFormBootstrap.php');

/*
 * Form will be called
 * a) with a SIP identifier, or
 * b) without a SIP identifier (form setting has to allow this) and will create on the fly a new SIP.
 *
 * The SIP-Store stores:
 *  form=<formname>
 *  r=<record id>  (table.id for a single record form)
 *  keySemId,keySemIduser
 *  <further individual variables>
 */

/**
 * Class Form
 * @package qfq
 */
class Form {

    /**
     * @var \qfq\Store instantiated class
     */
    protected $store = null;
    /**
     * @var Database instantiated class
     */
    protected $db = null;
    /**
     * @var Evaluate instantiated class
     */
    protected $eval = null;

    protected $formSpec = array();  // Form Definition: copy of the loaded form
    protected $feSpecAction = array(); // FormEelement Definition: all formElement.class='action' of the loaded form
    protected $feSpecNative = array(); // FormEelement Definition: all formElement.class='native' of the loaded form

    /*
     * TODO:
     *  Preparation: setup logging, database access, record locking
     *  fill stores
     *  Check permission_create / permission_update
     *  Multi: iterate over all records, Single: activate record
     *      Check mode: Load | Save
     *      doActions 'Before'
     *      Do all FormElements
     *      doActions 'After'
     */


//    protected $formElements = null;
//    protected $userLog = null;

    /**
     * Construct the Form Class and Store too. This is the base initialization moment.
     *
     * As a result of instantiating of Form, the class Store will initially called the first time and therefore instantiated automatically.
     * Store might throw an exception, in case the URL-passed SIP is invalid.
     *
     * @param string $bodytext
     */
    public function __construct($bodytext = '') {

        try {
            $this->store = Store::getInstance($bodytext);
            $this->db = new Database();
            $this->eval = new Evaluate($this->store, $this->db);
        } catch (UserException $e) {
            echo $e->formatMessage();
            exit;
        } catch (CodeException $e) {
            echo $e->formatMessage();
            exit;
        } catch (DbException $e) {
            echo $e->formatMessage();
            exit;
        } catch (\Exception $e) {
            echo "Generic Exception: " . $e->getMessage();
            exit;
        }


    }

    /**
     * @return string
     */
    public function process() {
        $html = '';
        $build = null;
        $master = null;

        mb_internal_encoding("UTF-8");

//        render:
//        plain
//        multimode: none

        try {

            $formName = $this->loadFormSpecification();

            $sipFound = $this->validateForm();
            if (!$sipFound) {
                $this->store->createSipAfterFormLoad($formName);
            }

            $this->store->fillStoreTableDefaultColumnType($this->formSpec['tableName']);

            switch ($this->formSpec['render']) {
                case 'plain':
                    $build = new BuildFormPlain($this->formSpec, $this->feSpecAction, $this->feSpecNative);
                    break;
                case 'table':
                    $build = new BuildFormTable($this->formSpec, $this->feSpecAction, $this->feSpecNative);
                    break;
                case 'bootstrap':
                    $build = new BuildFormBootstrap($this->formSpec, $this->feSpecAction, $this->feSpecNative);
                    break;
                default:
                    throw new CodeException("This statement should never be reached", ERROR_CODE_SHOULD_NOT_HAPPEN);
            }

            // Form action: load or save?
            $mode = isset($_SERVER['REQUEST_METHOD']) && $_SERVER['REQUEST_METHOD'] === 'POST' ? FORM_SAVE : FORM_LOAD;
            switch ($mode) {
                case FORM_LOAD:
                    $html .= $build->process();
                    break;
                case FORM_SAVE:
                    break;
                default:
                    throw new CodeException("This statement should never be reached", ERROR_CODE_SHOULD_NOT_HAPPEN);
            }
            return $html;

        } catch (UserException $e) {
            echo $e->formatMessage();
        } catch (CodeException $e) {
            echo $e->formatMessage();
        } catch (DbException $e) {
            echo $e->formatMessage();
        } catch (\Exception $e) {
            echo "Generic Exception: " . $e->getMessage();
        }

        return $html;
    }

    /**
     * Loads specification of recent form.
     * Evaluates Form and all FormElements.
     *
     * Loaded Form is in  $this->formSpec
     * Loaded 'action' FormElements are in $this->feSpecAction
     * Loaded 'native' FormElements are in $this->feSpecNative
     *
     * @return string formName
     * @throws DbException
     * @throws UserException
     */
    private function loadFormSpecification() {

        $formName = $this->getFormName();
        $this->store->setVar(SYSTEM_FORM, $formName, STORE_SYSTEM);

        $this->formSpec = $this->eval->parseArray($this->db->sql("SELECT * FROM Form AS f WHERE f.name LIKE ? AND f.deleted='no'", ROW_EXACT_1, [$formName], 'Form not found or multiple forms with the same name.'));
        HelperFormElement::explodeFieldParameter($this->formSpec);

        // Clear
        $this->store->setVar(SYSTEM_FORM_ELEMENT, '', STORE_SYSTEM);

        // "SELECT *, ? AS 'nestedInFieldSet' FROM FormElement AS fe WHERE fe.formId = ? AND fe.deleted = 'no' AND FIND_IN_SET(fe.class, ? ) AND fe.feIdContainer = ? AND fe.enabled='yes' ORDER BY fe.ord, fe.id";
        $sql = SQL_FORM_ELEMENT;

        // FE: Action
        $this->feSpecAction = $this->eval->parseArray($this->db->sql($sql, ROW_REGULAR, ['no', $this->formSpec["id"], 'action', 0]));
        HelperFormElement::explodeFieldParameter($this->feSpecAction);

        // FE: Native & Container
        $this->feSpecNative = $this->db->sql($sql, ROW_REGULAR, ['no', $this->formSpec["id"], 'native,container', 0]);
        HelperFormElement::explodeFieldParameter($this->feSpecNative);

        return $formName;
    }

    /**
     * @return string
     * @throws UserException
     * @throws CodeException
     */

    private function getFormName() {

        $formName = $this->store->getVar(SIP_FORM, STORE_TYPO3 . STORE_SIP . STORE_CLIENT);
        if ($formName !== false)
            return $formName;

        // TODO: phpunit tests all - missing formname always fires this exception
        throw new UserException("Missing form name. Not found in T3_BODYTEXT / SIP / GET", ERROR_MISSING_FORM_NAME);
    }

    /**
     * @return bool - 'true' if SIP exists, else 'false'
     * @throws CodeException
     * @throws UserException
     */
    private function validateForm() {

        // Retrieve record_id either from SIP (prefered) or via URL
        $r = $this->store->getVar(SIP_RECORD_ID, STORE_SIP . STORE_CLIENT);

        // If there is a record_id>0: EDIT else NEW
        $mode = ($r > 0) ? $this->formSpec['permitEdit'] : $this->formSpec['permitNew'];

        $feUserLoggedIn = isset($GLOBALS["TSFE"]->fe_user->user["uid"]) && $GLOBALS["TSFE"]->fe_user->user["uid"] > 0;

        $sipFound = $this->store->getVar(SIP_SIP, STORE_SIP) !== false;

        switch ($mode) {
            case  FORM_PERMISSION_SIP:
                if (!$sipFound) {
                    throw new UserException("SIP Parameter needed for this form.", ERROR_SIP_NEEDED_FOR_THIS_FORM);
                }
                break;
            case  FORM_PERMISSION_LOGGED_IN:
                if (!$feUserLoggedIn) {
                    throw new UserException("User not logged in.", ERROR_USER_NOT_LOGGED_IN);
                }
                break;
            case FORM_PERMISSION_LOGGED_OUT:
                if ($feUserLoggedIn) {
                    throw new UserException("User logged in.", ERROR_USER_LOGGED_IN);
                }
                break;
            case FORM_PERMISSION_ALWAYS:
                break;
            case FORM_PERMISSION_NEVER:
                throw new UserException("Loading form forbidden.", ERROR_FORM_FORBIDDEN);
            default:
                throw new CodeException("Unknown permission mode: '" . $mode . "'", ERROR_FORM_UNKNOWN_PERMISSION_MODE);
        }

        // Form Definition valid?
        if ($this->formSpec['multiMode'] !== 'none' && $this->formSpec['multiSql'] === '') {
            throw new UserException("MultiMode selected, but MultiSQL missing", ERROR_MULTI_SQL_MISSING);
        }

        return $sipFound;
    }

}