DROP TABLE IF EXISTS `Form`;
CREATE TABLE IF NOT EXISTS `Form` (
  `id`                       INT(11)                                                   NOT NULL  AUTO_INCREMENT,
  `name`                     VARCHAR(255)                                              NOT NULL  DEFAULT '',
  `title`                    VARCHAR(255)                                              NOT NULL  DEFAULT '',
  `noteInternal`             TEXT                                                      NOT NULL,
  `tableName`                VARCHAR(255)                                              NOT NULL  DEFAULT '',


  `permitNew`                ENUM('sip', 'logged_in', 'logged_out', 'always', 'never') NOT NULL  DEFAULT 'sip',
  `permitEdit`               ENUM('sip', 'logged_in', 'logged_out', 'always', 'never') NOT NULL  DEFAULT 'sip',
  `permitUrlParameter`       VARCHAR(255)                                              NOT NULL  DEFAULT '',
  `render`                   ENUM('plain', 'table', 'bootstrap')                       NOT NULL  DEFAULT 'plain',
  `multiMode`                ENUM('none', 'horizontal', 'vertical')                    NOT NULL  DEFAULT 'none',
  `multiSql`                 TEXT                                                      NOT NULL,
  `multiDetailForm`          VARCHAR(255)                                              NOT NULL  DEFAULT '',
  `multiDetailFormParameter` VARCHAR(255)                                              NOT NULL  DEFAULT '',

  `forwardMode`              ENUM('auto', 'no', 'page')                                NOT NULL  DEFAULT 'auto',
  `forwardPage`              VARCHAR(255)                                              NOT NULL  DEFAULT '',

  `bsLabelColumns`           VARCHAR(255)                                              NOT NULL  DEFAULT '',
  `bsInputColumns`           VARCHAR(255)                                              NOT NULL  DEFAULT '',
  `bsNoteColumns`            VARCHAR(255)                                              NOT NULL  DEFAULT '',

  `parameter` TEXT NOT NULL,

  `deleted`                  ENUM('yes', 'no')                                         NOT NULL  DEFAULT 'no',
  `modified`                 TIMESTAMP                                                 NOT NULL  DEFAULT CURRENT_TIMESTAMP,
  `created`                  DATETIME                                                  NOT NULL  DEFAULT '0000-00-00 00:00:00',

  PRIMARY KEY (`id`),
  KEY `name` (`name`),
  KEY `name_deleted` (`name`, `deleted`)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8
  AUTO_INCREMENT = 0;

--
-- Triggers `Form`
--
DROP TRIGGER IF EXISTS `on_Form_update_modified`;
DELIMITER //
CREATE TRIGGER `on_Form_update_modified` BEFORE UPDATE ON `Form`
FOR EACH ROW SET NEW.modified =
current_timestamp()
//
DELIMITER ;

# ----------------------------------------
# FormElement

DROP TABLE IF EXISTS `FormElement`;
CREATE TABLE IF NOT EXISTS `FormElement` (
  `id`            INT(11)                                                  NOT NULL                                                                                                                                                                                                                                                                                                                                                      AUTO_INCREMENT,
  `formId`        INT(11)                                                  NOT NULL,
  `feIdContainer` INT(11)                                                  NOT NULL DEFAULT '0',

  `enabled`       ENUM('yes', 'no')                                        NOT NULL DEFAULT 'yes',

  `name`          VARCHAR(255)                                             NOT NULL      DEFAULT '',
  `label`         VARCHAR(255)                                             NOT NULL      DEFAULT '',

  `mode`          ENUM('show', 'readonly', 'required', 'lock', 'disabled') NOT NULL DEFAULT 'show',
  `class`         ENUM('native', 'action', 'container')                    NOT NULL DEFAULT 'native',
  `type` ENUM('checkbox', 'dateJQW', 'datetimeJQW', 'gridJQW', 'hidden', 'input',
                       'note', 'password', 'radio', 'select', 'subrecord', 'upload', 'fieldset', 'pill',
                       'before_load', 'before_save', 'before_insert', 'before_update', 'before_delete', 'after_load',
                       'after_save', 'after_insert', 'after_update', 'after_delete', 'feGroup',
                       'sendmail')                                         NOT NULL DEFAULT 'input',
  `checkType`     ENUM('', 'min|max', 'pattern', 'number', 'email'),
  `checkPattern`  VARCHAR(255)                                             NOT NULL DEFAULT '',

  `onChange`      VARCHAR(255)                                             NOT NULL DEFAULT '',

  `ord`           INT(11)                                                  NOT NULL DEFAULT '0',
  `tabindex`      INT(11)                                                  NOT NULL DEFAULT '0',


  `size`          VARCHAR(255)                                             NOT NULL DEFAULT '',
  `maxLength`     VARCHAR(255)                                             NOT NULL DEFAULT '',
  `note`          TEXT                                                     NOT NULL,
  `tooltip`       VARCHAR(255)                                             NOT NULL DEFAULT '',
  `placeholder`   VARCHAR(255)                                             NOT NULL DEFAULT '',

  `value`         TEXT                                                     NOT NULL,
  `sql1`          TEXT                                                     NOT NULL,
  `sql2`          TEXT                                                     NOT NULL,
  `parameter`     TEXT                                                     NOT NULL,
  `clientJs` TEXT,

  `feGroup`       VARCHAR(255)                                             NOT NULL DEFAULT '',
  `debug`    ENUM('yes', 'no') NOT NULL DEFAULT 'no',
  `deleted`       ENUM('yes', 'no')                                        NOT NULL DEFAULT 'no',
  `modified`      TIMESTAMP                                                NOT NULL DEFAULT CURRENT_TIMESTAMP,
  `created`       DATETIME                                                 NOT NULL DEFAULT '0000-00-00 00:00:00',

  PRIMARY KEY (`id`),
  KEY `formId` (`formId`),
  KEY `formId_class_enabled_deleted` (`formId`, `class`, `enabled`, `deleted`)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8
  AUTO_INCREMENT = 0;

--
-- Triggers `FormElement`
--
DROP TRIGGER IF EXISTS `on_FormEelement_update_modified`;
DELIMITER //
CREATE TRIGGER `on_FormEelement_update_modified` BEFORE UPDATE ON `FormElement`
FOR EACH ROW SET NEW.modified =
current_timestamp()
//
DELIMITER ;


#
# FormEditor: Form
INSERT INTO Form (name, title, noteInternal, tableName, permitNew, permitEdit, render, multiSql, parameter) VALUES
  ('form', 'Form Editor: {{SELECT id, " / ", title FROM Form WHERE id = {{recordId:SZ}}}}', 'Please secure the form',
   'Form', 'always', 'always', 'bootstrap', '', 'maxVisiblePill=3');

# FormEditor: FormElements
INSERT INTO FormElement (formId, name, label, mode, class, type, ord, size, maxLength, note, clientJs, value, sql1, sql2, parameter, feIdContainer)
VALUES
  (1, 'basic', 'Basic', 'show', 'pill', 'container', 10, 0, 0, '', '', '', '', '', '', 0),
  (1, 'permission', 'Permission', 'show', 'pill', 'container', 20, 0, 0, '', '', '', '', '', '', 0),
  (1, 'various', 'Various', 'show', 'pill', 'container', 30, 0, 0, '', '', '', '', '', '', 0),
  (1, 'formelement', 'Formelement', 'show', 'pill', 'container', 40, 0, 0, '', '', '', '', '', '', 0),

  (1, 'id', 'id', 'readonly', 'input', 'native', 100, 10, 11, '', '', '', '', '', '', 1),
  (1, 'name', 'Name', 'show', 'input', 'native', 120, 40, 255, '', '', '', '', '', 'autofocus=on', 1),
  (1, 'title', 'Title', 'show', 'input', 'native', 130, 40, 255, '', '', '', '', '', '', 1),
  (1, 'noteInternal', 'Note', 'show', 'input', 'native', 140, '40,3', 0, '', '', '', '', '', '', 1),
  (1, 'tableName', 'Table', 'required', 'select', 'native', 150, 0, 0, '', '', '', '{{!SHOW tables}}', '', 'emptyItemAtStart', 1),

  (1, 'permitNew', 'Permit New', 'show', 'radio', 'native', 160, 0, 0, '', '', '', '', '', '', 2),
  (1, 'permitEdit', 'Permit Edit', 'show', 'radio', 'native', 170, 0, 0, '', '', '', '', '', '', 2),
  (1, 'permitUrlParameter', 'Permit Url Parameter', 'show', 'input', 'native', 180, 40, 255, '', '', '', '', '', '', 2),
  (1, 'render', 'Render', 'show', 'radio', 'native', 190, 0, 0, '', '', '', '', '', '', 2),

  (1, 'multi', 'Multi', 'show', 'fieldset', 'native', 210, 0, 0, '', '', '', '', '', '', 3),
  (1, 'multiMode', 'Multi Mode', 'show', 'radio', 'native', 220, 0, 0, '', '', '', '', '', '', 3),
  (1, 'multiSql', 'Multi SQL', 'show', 'input', 'native', 230, '40,3', 0, '', '', '', '', '', '', 3),
  (1, 'multiDetailForm', 'Multi Detail Form', 'show', 'input', 'native', 240, 40, 255, '', '', '', '', '', '', 3),
  (1, 'multiDetailFormParameter', 'Multi Detail Form Parameter', 'show', 'input', 'native', 250, 40, 255, '', '', '', '', '', '', 3),
  (1, 'forwardMode', 'Forward', 'show', 'radio', 'native', 260, 0, 0, '', '', '', '', '', '', 3),
  (1, 'forwardPage', 'Forward Page', 'show', 'input', 'native', 270, 40, 255, '', '', '', '', '', '', 3),
  (1, 'bsLabelColumns', 'BS Label Columns', 'show', 'input', 'native', 280, 40, 250, '', '', '', '', '', '', 3),
  (1, 'bsInputColumns', 'BS Input Columns', 'show', 'input', 'native', 290, 40, 250, '', '', '', '', '', '', 3),
  (1, 'bsNoteColumns', 'BS Note Columns', 'show', 'input', 'native', 300, 40, 250, '', '', '', '', '', '', 3),

  (1, 'deleted', 'Deleted', 'show', 'checkbox', 'native', 400, 0, 0, '', '', '', '', '', '', 3),
  (1, 'modified', 'Modified', 'readonly', 'input', 'native', 410, 40, 20, '', '', '', '', '', '', 3),
  (1, 'created', 'Created', 'readonly', 'input', 'native', 420, 40, 20, '', '', '', '', '', '', 3),

  (1, '', 'FormElements', 'show', 'subrecord', 'native', 500, 0, 0, '', '', '',
   '{{!SELECT * FROM FormElement WHERE formId={{id:R0}}}}',
   '', 'form=formElement\npage=form.php', 4);


#
# FormEditor: FormElement
INSERT INTO Form (name, title, noteInternal, tableName, permitNew, permitEdit, render, multiSql, parameter) VALUES
  ('formElement', 'Form Element Editor : {{SELECT id, " / ", title FROM Form WHERE id = {{recordId:SZ}}}}',
   'Please secure the form',
   'FormElement', 'always', 'always', 'bootstrap', '', 'maxVisiblePill=3');

# FormEditor: FormElements
INSERT INTO FormElement (id, formId, name, label, mode, type, ord, size, maxLength, note, clientJs, value, sql1, sql2, parameter, feIdContainer, debug)
VALUES

  (100, 2, 'basic', 'Basic', 'show', 'pill', 10, 0, 0, '', '', '', '', '', '', 0, 'no'),
  (101, 2, 'check_order', 'Check & Order', 'show', 'pill', 20, 0, 0, '', '', '', '', '', '', 0, 'no'),
  (102, 2, 'layout', 'Layout', 'show', 'pill', 20, 0, 0, '', '', '', '', '', '', 0, 'no'),
  (103, 2, 'value', 'Value', 'show', 'pill', 20, 0, 0, '', '', '', '', '', '', 0, 'no'),
  (104, 2, 'info', 'Info', 'show', 'pill', 20, 0, 0, '', '', '', '', '', '', 0, 'no'),

  (110, 2, 'id', 'id', 'readonly', 'input', 100, 10, 11, '', '', '', '', '', '', 100, 'no'),
  (111, 2, 'formId', 'formId', 'readonly', 'input', 120, 40, 255, '', '', '', '', '', '', 100, 'no'),
  (112, 2, 'feIdContainer', 'Container', 'show', 'select', 150, 0, 0, '', '', '',
   '{{!SELECT fe.id, CONCAT(fe.class, " / ", fe.label) FROM FormElement As fe WHERE fe.formId={{id}} AND fe.class="container" ORDER BY fe.ord }}',
   '', 'emptyItemAtStart', 100, 'no'),
  (113, 2, 'enabled', 'Enabled', 'show', 'checkbox', 120, 0, 0, '', '', '', '', '', '', 100, 'no'),
  (114, 2, 'name', 'Name', 'show', 'input', 120, 40, 255, '', '', '', '', '', '', 100, 'no'),
  (115, 2, 'label', 'Label', 'show', 'input', 130, 40, 255, '', '', '', '', '', '', 100, 'no'),
  (116, 2, 'mode', 'Mode', 'show', 'select', 120, 0, 255, '', '', '', '', '', '', 100, 'no'),
  (117, 2, 'class', 'Class', 'show', 'select', 120, 0, 255, '', '', '', '', '', '', 100, 'no'),
  (118, 2, 'type', 'Type', 'show', 'select', 120, 0, 255, '', '', '', '', '', '', 100, 'no'),
  (119, 2, 'checkType', 'Check Type', 'show', 'select', 120, 0, 255, '', '', '', '', '', '', 101, 'no'),
  (120, 2, 'checkPattern', 'Check Pattern', 'show', 'input', 130, 40, 255, '', '', '', '', '', '', 101, 'no'),
  (121, 2, 'onChange', 'JS onChange', 'show', 'input', 130, 40, 255, '', '', '', '', '', '', 101, 'no'),
  (122, 2, 'ord', 'Order', 'show', 'input', 130, 40, 255, '', '', '', '', '', '', 101, 'no'),
  (123, 2, 'tabindex', 'tabindex', 'show', 'input', 130, 40, 255, '', '', '', '', '', '', 101, 'no'),
  (124, 2, 'size', 'Size', 'show', 'input', 130, 40, 255, '', '', '', '', '', '', 102, 'no'),
  (125, 2, 'maxlenght', 'Maxlength', 'show', 'input', 130, 40, 255, '', '', '', '', '', '', 102, 'no'),
  (126, 2, 'note', 'note', 'show', 'input', 130, 40, 255, '', '', '', '', '', '', 102, 'no'),
  (127, 2, 'tooltip', 'Tooltip', 'show', 'input', 130, 40, 255, '', '', '', '', '', '', 102, 'no'),
  (128, 2, 'placeholder', 'Placeholder', 'show', 'input', 130, 40, 255, '', '', '', '', '', '', 102, 'no'),
  (129, 2, 'value', 'value', 'show', 'input', 130, 40, 255, '', '', '', '', '', '', 102, 'no'),
  (130, 2, 'sql1', 'sql1', 'show', 'input', 130, '40,4', 255, '', '', '', '', '', '', 103, 'no'),
  (131, 2, 'parameter', 'Parameter', 'show', 'input', 130, '40,4', 255, '', '', '', '', '', '', 103, 'no'),
  (132, 2, 'clientJs', 'ClientJS', 'show', 'input', 130, 40, 255, '', '', '', '', '', '', 103, 'no'),
  (133, 2, 'feGroup', 'feGroup', 'show', 'input', 130, 40, 255, '', '', '', '', '', '', 104, 'no'),
  (134, 2, 'debug', 'Debug', 'show', 'checkbox', 130, 0, 0, '', '', '', '', '', '', 104, 'no'),
  (135, 2, 'deleted', 'Deleted', 'show', 'checkbox', 400, 0, 0, '', '', '', '', '', '', 104, 'no'),
  (136, 2, 'modified', 'Modified', 'readonly', 'input', 410, 40, 20, '', '', '', '', '', '', 104, 'no'),
  (137, 2, 'created', 'Created', 'readonly', 'input', 420, 40, 20, '', '', '', '', '', '', 104, 'no');



